@@ -112,14 +112,18 @@ CLIENT_LOGIN_SCOPES = {
         'http://picasaweb.google.com/data/'],
     'apps': [ # Google Apps Provisioning API
         'http://www.google.com/a/feeds/',
-        'https://www.google.com/a/feeds/'],
+        'https://www.google.com/a/feeds/',
+        'http://apps-apis.google.com/a/feeds/',
+        'https://apps-apis.google.com/a/feeds/'],
     'wise': [ # Spreadsheets Data API
         'https://spreadsheets.google.com/feeds/',
         'http://spreadsheets.google.com/feeds/'],
     'sitemaps': [ # Google Webmaster Tools
         'https://www.google.com/webmasters/tools/feeds/'],
     'youtube': [ # YouTube
-        'http://gdata.youtube.com/feeds/api/']}
+        'http://gdata.youtube.com/feeds/api/',
+        'http://uploads.gdata.youtube.com/feeds/api',
+        'http://gdata.youtube.com/action/GetUploadToken']}
 
 
 def lookup_scopes(service_name):
@@ -247,6 +251,10 @@ class GDataService(atom.service.AtomService):
     self.__captcha_token = None
     self.__captcha_url = None
     self.__gsessionid = None
+
+    if http_request_handler.__name__ == 'gdata.urlfetch':
+      import gdata.alt.appengine
+      self.http_client = gdata.alt.appengine.AppEngineHttpClient()
  
   # Define properties for GDataService
   def _SetAuthSubToken(self, auth_token, scopes=None):
@@ -263,7 +271,7 @@ class GDataService(atom.service.AtomService):
     Returns:
       string
     """
-    current_scopes = lookup_scopes(self.service, use_default=False)
+    current_scopes = lookup_scopes(self.service)
     if current_scopes:
       token = self.token_store.find_token(current_scopes[0])
       if hasattr(token, 'auth_header'):
@@ -305,18 +313,26 @@ class GDataService(atom.service.AtomService):
       doc="""Get the captcha URL for a login request.""")
 
   def GetAuthSubToken(self):
-    """Returns the AuthSub Token after removing the AuthSub Authorization
-    Label.
+    """Returns the AuthSub token string (after removing the AuthSub 
+    Authorization Label).    
      
     The AuthSub Authorization Label reads: "AuthSub token"
 
+    This method examines the current_token to see if it is an AuthSubToken.
+    If not, it searches the token_store for a token which matches the 
+    current scope.
+    
+    The current scope is determined by the service name string member.
+    
     Returns:
-      If the AuthSub Token is set AND it begins with the AuthSub 
-      Authorization Label, the AuthSub Token is returned minus the AuthSub
-      Label. If the AuthSub Token does not start with the AuthSub
-      Authorization Label or it is not set, None is returned.
+      If the current_token is set to an AuthSubToken, return the token
+      string. If there is no current_token, a token string for a token
+      which matches the service object's default scope is returned. If
+      there are no tokens valid for the scope, returns None.
     """
-    current_scopes = lookup_scopes(self.service)#, use_default=False)
+    if isinstance(self.current_token, gdata.auth.AuthSubToken):
+      return self.current_token.get_token_string()
+    current_scopes = lookup_scopes(self.service)
     if current_scopes:
       token = self.token_store.find_token(current_scopes[0])
       if isinstance(token, gdata.auth.AuthSubToken):
@@ -330,8 +346,10 @@ class GDataService(atom.service.AtomService):
   def SetAuthSubToken(self, token, scopes=None):
     """Sets the token sent in requests to an AuthSub token.
 
+    Sets the current_token and attempts to add the token to the token_store.
+    
     Only use this method if you have received a token from the AuthSub
-    service. The authi token is set automatically when UpgradeToSessionToken()
+    service. The auth token is set automatically when UpgradeToSessionToken()
     is used. See documentation for Google AuthSub here:
     http://code.google.com/apis/accounts/AuthForWebApps.html 
 
@@ -358,15 +376,26 @@ class GDataService(atom.service.AtomService):
         if scopes is None:
           scopes = [atom.token_store.SCOPE_ALL]
       token.scopes = scopes
-    self.token_store.add_token(token)
+    if self.auto_set_current_token:
+      self.current_token = token
+    if self.auto_store_tokens:
+      self.token_store.add_token(token)
 
   def GetClientLoginToken(self):
-    """Returns the token string for the current request scope.
-  
+    """Returns the token string for the current token or a token matching the 
+    service scope.
+
+    If the current_token is a ClientLoginToken, the token string for 
+    the current token is returned. If the current_token is not set, this method
+    searches for a token in the token_store which is valid for the service 
+    object's current scope.
+
     The current scope is determined by the service name string member.
     The token string is the end of the Authorization header, it doesn not
     include the ClientLogin label.
     """
+    if isinstance(self.current_token, gdata.auth.ClientLoginToken):
+      return self.current_token.get_token_string()
     current_scopes = lookup_scopes(self.service)
     if current_scopes:
       token = self.token_store.find_token(current_scopes[0])
@@ -379,15 +408,18 @@ class GDataService(atom.service.AtomService):
       return None
 
   def SetClientLoginToken(self, token, scopes=None):
-    """Sets the token sent in requests to an ClientLogin token.
+    """Sets the token sent in requests to a ClientLogin token.
 
+    This method sets the current_token to a new ClientLoginToken and it 
+    also attempts to add the ClientLoginToken to the token_store.
+    
     Only use this method if you have received a token from the ClientLogin
     service. The auth_token is set automatically when ProgrammaticLogin()
     is used. See documentation for Google ClientLogin here:
     http://code.google.com/apis/accounts/docs/AuthForInstalledApps.html
 
     Args:
-      token: string The token returned by the ClientLogin service.
+      token: string or instance of a ClientLoginToken. 
     """
     if not isinstance(token, gdata.auth.ClientLoginToken):
       token_string = token
@@ -400,7 +432,10 @@ class GDataService(atom.service.AtomService):
         if scopes is None:
           scopes = [atom.token_store.SCOPE_ALL]
       token.scopes = scopes
-    self.token_store.add_token(token)
+    if self.auto_set_current_token:
+      self.current_token = token
+    if self.auto_store_tokens:
+      self.token_store.add_token(token)
 
   # Private methods to create the source property.
   def __GetSource(self):
@@ -935,9 +970,9 @@ class GDataService(atom.service.AtomService):
           m = re.compile('[\?\&]gsessionid=(\w*)').search(location)
           if m is not None:
             self.__gsessionid = m.group(1) 
-          return GDataService.Post(self, data, location, extra_headers, 
-              url_params, escape_params, redirects_remaining - 1, 
-              media_source, converter=converter)
+          return GDataService.PostOrPut(self, verb, data, location, 
+              extra_headers, url_params, escape_params, 
+              redirects_remaining - 1, media_source, converter=converter)
         else:
           raise RequestError, {'status': server_response.status,
               'reason': '302 received without Location header',
